Ext.define('AS.ARC.apps.vpnServer.core.addUserButton', {
	extend: 'Ext.button.Button',

	alias: ['widget.asVpnAddUserButton'],

	text: _S('COMMON', 'ADD'),

	initComponent: function () {
		var me = this;		

		me.callParent(arguments);

	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);
	},

	handler: function (me) {
		var page = me.page;

		var accFilter = Ext.create('AS.ARC.asSearchField', {
			listeners: {
				change: function (filter, newValue) {
					accGrid.store.clearFilter();
					if (newValue !== '') {
						accGrid.store.filter([
							{
								filterFn: function (item) {
									var data = item.data;
									if (data.name.indexOf(newValue) !== -1)
										return true;
									if (data.status.indexOf(newValue) !== -1)
										return true;
								}
							}
						]);
					} else {
						getUniqueAccount(accStore, me.grid);
					}
				}
			}
		});

		var api,
			urlOpt;

		if (me.action === 'user') {
			api = AS.ARC.util.getApiUrl('user.cgi', 'accessControl');
		} else {
			api = AS.ARC.util.getApiUrl('group.cgi', 'accessControl');
		}

		urlOpt = {
			act: 'list',
			domain: 0
		};

		var getUniqueAccount = function (store, viewGrid) {
			store.filter([{
				filterFn: function (item) {
					var isValid = true;
					viewGrid.store.each(function (r) {
						if (r.data.name === item.data.name) {
							isValid = false;
							return false;
						}
					});
					return isValid;
				}
			}]);
		};

		var accStore = Ext.create(Ext.data.Store, {
			pageSize: 10000,
			autoLoad: true,
			fields: ['name', 'status'],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(api, urlOpt),
				actionMethods: {
					create: 'POST',
					read: 'POST',
					update: 'POST',
					destroy: 'POST'
				},
				reader: {
					type: 'json',
					root: 'datas'
				}
			},
			listeners: {
				beforeload: function () {
					addWin.el.mask(_S('COMMON', 'LOADING'));
				},
				load: function (store) {
					//grid.filter([{
					//	filterFn: function (item) {
					//		var isValid = true;
					//		me.grid.store.each(function (r) {
					//			if (r.data.name === item.data.name) {
					//				isValid = false;
					//				return false;
					//			}
					//		});
					//		return isValid;
					//	}
					//}]);
					getUniqueAccount(store, me.grid);
					addWin.el.unmask();
				},
				exception: function (proxy, response, operation, opts) {
					AS.ARC.util.showMsgWindow({
					}, Ext.JSON.decode(response.responseText), fn.win);
				}
			}
		});

		var check = Ext.create('AS.ARC.checkColumn', {
			//enableSingleSelect: true,
			menuDisabled: true,
			align: 'center',
			sortable: false,
			dataIndex: '',
			width: 25
		});

		var accGrid = Ext.create(Ext.grid.Panel, {
			itemId: 'grid',			
			width: '100%',
			height: 190,
			bodyStyle: 'border: none;',
			columns: [
				check,
				{
					dataIndex: 'name',
					flex: 2,
					//align: 'center',
					header: _S('COMMON', 'NAME'),
					menuDisabled: true,
					renderer: function (value, metadata) {
						metadata.tdAttr = 'data-qtip="' + value + '"';						
						return value;
					}
				},
				{
					dataIndex: 'status',
					flex: 1,
					align: 'center',
					header: _S('COMMON', 'STATUS'),
					menuDisabled: true,
					hidden: (me.action === 'user') ? false : true,
					renderer: function (value, metadata) {
						metadata.tdAttr = 'data-qtip="' + value + '"';
						if (value.toLowerCase() === 'active') {
							value = _S('COMMON', 'ACTIVE');
						} else {
							value = _S('COMMON', 'INACTIVE');
						}
						return value;
					}
				}			
			],
			store: accStore			
		});		
		
		var addWin = Ext.create('AS.ARC.subWindow', {
			title: me.title,
			layout: 'fit',
			parentWin: page.fn.app.win,
			width: 450,
			height: 360,
			fbar: [
				{
					text: _S('COMMON', 'SAVE'),
					handler: function () {
						addWin.el.mask(_S('COMMON', 'LOADING'));

						var name = [];

						Ext.each(check.getSelection(), function (data) {
							name.push({
								name: data.name,
								pptp: false,
								openvpn: false
							});							
						});						
					
						AS.ARC.ajax({
							url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
								act: 'set',
								section: 'privilege',
								tab: (me.action === 'user') ? 'user' : 'group'
							}),
							//params: Ext.JSON.encode({ priv: name }),
							params: {
								priv: Ext.JSON.encode(name)
							},
							method: 'post',
							success: function (json, opts) {
								//addWin.el.unmask();
								me.grid.store.load()
								addWin.close();
							},
							failure: function (json, opts) {
								//addWin.el.unmask();
								AS.ARC.util.showMsgWindow({

								}, json, addWin);
							}
						});
					}
				},
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function () {
						addWin.close();
					}
				}],
			items: [
				{
					xtype: 'asPanelTop10',
					items: [						
						{
							xtype: 'asPanelToolbarTop',
							items: [
								accFilter
							]
						},
						accGrid
					]
				}
			]
		}).show();
	}
});

Ext.define('AS.ARC.apps.vpnServer.core.privFormPanel', {
	extend: 'Ext.container.Container', // 沒有使用到 asForm 的功能所以改為 Container

	alias: ['widget.asVpnPrivilege'],
	store: '',

	initComponent: function () {
		var me = this,
			page = me.page;
		
		var title;

		if (me.action === 'user') {
			title = _S('VPN_SERVER', 'ADD_ALLOW_USER');
		} else {
			title = _S('VPN_SERVER', 'ADD_ALLOW_GROUP');
		}
		
		var pptpCheck = Ext.create('AS.ARC.checkColumn', {
			itemId: 'pptp-check',
			//enableSingleSelect: true,
			header: _S('VPN_SERVER', 'PPTP'),
			dataIndex: 'pptp',
			flex: 2,
			menuDisabled: true,
			align: 'center',
			sortable: false,
			width: 25
		});

		var vpnCheck = Ext.create('AS.ARC.checkColumn', {
			//enableSingleSelect: true,
			itemId: 'openvpn-check',
			header: _S('VPN_SERVER', 'OPENVPN'),
			dataIndex: 'openvpn',
			flex: 2,
			menuDisabled: true,
			align: 'center',
			sortable: false,
		});

		var xl2tpCheck = Ext.create('AS.ARC.checkColumn', {
			//enableSingleSelect: true,
			itemId: 'xl2tp-check',
			header: _S('VPN_SERVER', 'XL2TP'),
			dataIndex: 'xl2tp',
			flex: 2,
			menuDisabled: true,
			align: 'center',
			sortable: false
		});

		var grid = Ext.create(Ext.grid.Panel, {
			itemId: 'grid',
			cls: 'grid-priv',
			height: 350,
			width: '100%',
			isDirty: function () {
				return (pptpCheck.getDirtyIndex().length !== 0) || (vpnCheck.getDirtyIndex().length !== 0) || (xl2tpCheck.getDirtyIndex().length !== 0);
			},
            multiSelect: true,
			columns: [
				{
					dataIndex: 'name',
					flex: 3,
					align: 'center',
					header: _S('COMMON', 'NAME'),
					menuDisabled: true,
					renderer: function (value, metadata) {
						metadata.tdAttr = 'data-qtip="' + value + '"';
						return value;
					}
				},
				{
					dataIndex: 'status',
					hidden: (me.action === 'user') ? false : true,
					flex: 3,
					align: 'center',
					header: _S('COMMON', 'STATUS'),
					menuDisabled: true,
					renderer: function (value, metadata) {
						metadata.tdAttr = 'data-qtip="' + value + '"';
						if (me.action === 'user') {
							if (value.toLowerCase() === 'active') {
								value = _S('COMMON', 'ACTIVE');
							} else {
								value = _S('COMMON', 'INACTIVE');
							}
						}
						return value;
					}
				},
				pptpCheck,
				vpnCheck,
				xl2tpCheck
			],
			store: me.store,
			listeners: {
				selectionchange: function (me, selected) {
					if (selected.length === 0) {
						removeBtn.disable();
					} else {
						removeBtn.enable();
					}
				}
			}
		});

		var removeBtn = Ext.create(Ext.button.Button, {
			text: _S('COMMON', 'REMOVE'),
			disabled: true,
			handler: function () {
				page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				var sel = grid.getSelectionModel().getSelection(),
					name = [];
				sel.each(function (r) {
					name.push({ name: r.data.name });
				});
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'del',
						section: 'privilege',
						tab: (me.action === 'user') ? 'user' : 'group'
					}),
					//params: Ext.JSON.encode({ priv: name }),
					params: {
						priv: Ext.JSON.encode(name)
					},
					method: 'post',
					success: function (json, opts) {
						page.fn.app.unmaskWin();
						grid.store.load();
					},
					failure: function (json, opts) {
						page.fn.app.unmaskWin();
						AS.ARC.util.showMsgWindow({

						}, json, page.fn.app.win);
					}
				});				
			}
		});
		
		var search = Ext.create('AS.ARC.asSearchField', {
			listeners: {
				change: function (me, newValue) {
					grid.store.clearFilter();
					if (newValue !== '') {
						grid.store.filter([
							{
								filterFn: function (item) {
									var data = item.data;
									//if (data.time.indexOf(newValue) !== -1)
									//	return true;
									if (data.name.indexOf(newValue) !== -1)
										return true;
								}
							}
						]);
					}
				}
			}
		});

		Ext.apply(me, {
			items: [
				{
					xtype: 'asPanelToolbarTop',
					items: [
						{
							xtype: 'asVpnAddUserButton',
							action: me.action,
							grid: grid,
							page: page,
							title: title
						},
						removeBtn,
						search
					]
				},
				grid
			]
		});

		me.on('show', function () {
			changedLabel.resetDisplay();
		});

		me.callParent(arguments);
	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);
	},

	loadData: function () {
		var me = this;
		me.store.load();
	}
});

Ext.define('AS.ARC.apps.vpnServer.privilege.user', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'user',

	handleApply: function () {
		var page = this;

		var userData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(userData)) {
			page.fp.showSetNotChange();
			return false;
		}

		page.fn.app.maskWin(_S('COMMON', 'LOADING'));
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
				act: 'set',
				section: 'privilege',
				tab: 'user'
			}),
			params: userData,
			method: 'post',
			success: function (json, opts) {
				page.fp.showApply();
				page.fn.app.unmaskWin();
				page.userStore.load();
			},
			failure: function (json, opts) {						
				page.fn.app.unmaskWin();
				page.userStore.load();

				AS.ARC.util.showMsgWindow({
					5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
					5302: _S('ERROR', 'INCORRECT_PARAMETER'),
					5006: _S('ERROR', 'CANNOT_READ_CONFIG')
				}, json, fn.win);
			}
		});
	},

	getPanel: function () {
		var page = this;

        page.userStore = Ext.create(Ext.data.Store, {
			autoLoad: false,
			fields: ['name', 'pptp', 'openvpn', 'status', 'xl2tp'],
			sorters: [{ property: 'name', direction: 'ASC' }],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
					act: 'get',
					section: 'privilege',
					tab: 'user'
				}),
				actionMethods: {
					create: 'POST',
					read: 'POST',
					update: 'POST',
					destroy: 'POST'
				},
				reader: {
					type: 'json',
					root: 'user'
				}
			},
			listeners: {
				beforeload: function () {
					page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				},
				load: function (me) {
					page.fn.app.unmaskWin();

					page.fp.uiChecker.srcData = page.doDataExchange();
				},
				exception: function (proxy, response, operation, opts) {
					AS.ARC.util.showMsgWindow({
						5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
						5302: _S('ERROR', 'INCORRECT_PARAMETER'),
						5006: _S('ERROR', 'CANNOT_READ_CONFIG')
					}, Ext.JSON.decode(response.responseText), fn.win);
				}
			}
		});

        page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		page.doDataExchange = function () {
			var pageData = { priv: Ext.JSON.encode(page.userStore.getRange().map(function (d) { return d.data; })) };
			return pageData;
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title: _S('COMMON', 'USER'),
			border: false,
			cls: 'as-page-panel',
			items: [{
					xtype: 'asVpnPrivilege',
					itemId: 'user-tab',
					action: 'user',
					store: page.userStore,
					page: page,
                    border: 'none'
			}],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange,
			listeners: {
				resize: function( main, adjWidth, adjHeight, eOpts ) {
					var reHeight = main.up().getHeight();
					main.down('#grid').setHeight(reHeight - 210);
				}
			}
		};
	},

	loadData: function () {
		var page = this,
            userTopbar = page.panelItems.items[0].items.items[0],
            userGrid = page.panelItems.items[0].items.items[1],
            userFootbar = page.panelItems.items[0].items.items[2];

        userGrid.store.load();
	}
});

Ext.define('AS.ARC.apps.vpnServer.privilege.group', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'group',

	handleApply: function () {
		var page = this;

		var groupData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(groupData)) {
			page.fp.showSetNotChange();
			return false;
		}

		page.fn.app.maskWin(_S('COMMON', 'LOADING'));
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
				act: 'set',
				section: 'privilege',
				tab: 'group'
			}),
			params: groupData,
			method: 'post',
			success: function (json, opts) {
				page.fp.showApply();
				page.fn.app.unmaskWin();
				page.groupStore.load();
			},
			failure: function (json, opts) {						
				page.fn.app.unmaskWin();

				page.groupStore.load();

				AS.ARC.util.showMsgWindow({
					5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
					5302: _S('ERROR', 'INCORRECT_PARAMETER'),
					5006: _S('ERROR', 'CANNOT_READ_CONFIG')
				}, json, fn.win);
			}
		});
	},

	getPanel: function () {
		var page = this;

        page.groupStore = Ext.create(Ext.data.Store, {
			autoLoad: false,
			fields: ['name', 'pptp', 'openvpn', 'enable', 'xl2tp'],
			sorters: [{ property: 'name', direction: 'ASC' }],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
					act: 'get',
					section: 'privilege',
					tab: 'group'
				}),
				actionMethods: {
					create: 'POST',
					read: 'POST',
					update: 'POST',
					destroy: 'POST'
				},
				reader: {
					type: 'json',
					root: 'group'
				}
			},
			listeners: {
				beforeload: function () {
					page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				},
				load: function (me) {
					page.fn.app.unmaskWin();

					page.fp.uiChecker.srcData = page.doDataExchange();
				},
				exception: function (proxy, response, operation, opts) {
					AS.ARC.util.showMsgWindow({
						5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
						5302: _S('ERROR', 'INCORRECT_PARAMETER'),
						5006: _S('ERROR', 'CANNOT_READ_CONFIG')
					}, Ext.JSON.decode(response.responseText), fn.win);
				}
			}
		});

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		page.doDataExchange = function () {
			var pageData = { priv: Ext.JSON.encode(page.groupStore.getRange().map(function (d) { return d.data; })) };
			return pageData;
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title: _S('COMMON', 'GROUP'),
			border: false,
			cls: 'as-page-panel',
			items: [{
					xtype: 'asVpnPrivilege',
					itemId: 'group-tab',
					action: 'group',
					store: page.groupStore,
					page: page,
                    border: 'none'
			}],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange,
			listeners: {
				resize: function( main, adjWidth, adjHeight, eOpts ) {
					var reHeight = main.up().getHeight();
					main.down('#grid').setHeight(reHeight - 210);
				}
			}
		};
	},

	loadData: function () {
		var page = this,
            userTopbar = page.panelItems.items[0].items.items[0],
            groupGrid = page.panelItems.items[0].items.items[1],
            userFootbar = page.panelItems.items[0].items.items[2];

        groupGrid.store.load();
            
	}
});

/**
 * @class AS.ARC.apps.vpnServer.privilege.fn
 * @extends AS.ARC.apps.vpnServer.fn
 * <p></p>
 */
Ext.define('AS.ARC.apps.vpnServer.privilege.fn', {
	extend: 'AS.ARC.apps.vpnServer.fn',

	fnTag: 'privilege',
	
	init: function (config) {
		this.apiUrl = AS.ARC.util.getUserAppsPath() + 'vpn-server/vpnserver.cgi';
		this.imageUrl = AS.ARC.util.getUserAppsPath() + 'vpn-server/images/';
		
		this.createPanel();
	},

	createPanel: function () {
		var fn = this;
		
		var pageUser = new AS.ARC.apps.vpnServer.privilege.user({fn: this});
        var pageGroup = new AS.ARC.apps.vpnServer.privilege.group({fn: this});
			
		this.tabPanel = Ext.create('AS.ARC.fnTabPanel', {
			items: [
				pageUser.getPanel(),
                pageGroup.getPanel()
			],
			listeners: {
				tabchange: function(tabpanel) {
					var activeIndex = tabpanel.items.indexOf(tabpanel.getLayout().activeItem);
				}
			}
		});

		// fn.tabPanel.enableDataConfirm();
		fn.initMainPanel();
	},
	
	loadData: function () {
		this.tabPanel.activeTab.page.loadData(true);
	}
});