Ext.define('AS.ARC.apps.vpnServer.core.maxClientCombo', {
	extend: 'Ext.form.ComboBox',

	alias: ['widget.asVpnMaxClientCombo'],

	editable: false,

	queryMode: 'local',
	displayField: 'name',
	valueField: 'value',
	value: 5,

	initComponent: function () {
		var me = this;

		me.store = Ext.create(Ext.data.Store, {
			fields: ['name', 'value'],
			data: [
				{ name: 5, value: 5 },
				{ name: 10, value: 10 },
				{ name: 15, value: 15 },
				{ name: 20, value: 20 },
			]
		});

		me.callParent(arguments);

	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);
	}	
});

Ext.define('AS.ARC.apps.vpnServer.core.dynIPField', {
	extend: 'Ext.container.Container',

	alias: ['widget.asVpnDynIPField'],

	layout: {type: 'hbox', align: 'middle'},

	fieldDisabled: [],
	defaultAddress: [],

	origMinMaxConf: [
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 0, max: 255 }
	],

	destMinMaxConf: [
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 2, max: 254 }
	],		

	getSubnetRange: function () {
		var me = this,
			ipValue = me.getValue(),
			destLast = parseInt(ipValue[1].split('.')[3]),
			origLast = parseInt(ipValue[0].split('.')[3]);
		return destLast - origLast + 1;
	},

	initComponent: function () {
		var me = this;

		me.addEvents('rangechange');

		me.items = [
			{
				xtype: 'displayfield',
				fieldLabel: _S('VPN_SERVER', 'DYNAMIC_IP_ADDRESS'),
				labelSeparator: '',
				labelWidth: AS.ARC.config.layout.labelWidthMedium
			},
			{
				xtype: 'asVpnIPField',
				itemId: 'original',
				minMaxConf: me.origMinMaxConf,
				width: 180,
				listeners: {
					change: function (field) {
						var fieldItems = field.items,
							destFieldItems = field.up().items.get('destination').items;

						destFieldItems.get('ip-first').setValue(fieldItems.get('ip-first').getValue());
						destFieldItems.get('ip-second').setValue(fieldItems.get('ip-second').getValue());
						destFieldItems.get('ip-third').setValue(fieldItems.get('ip-third').getValue());

						destFieldItems.get('ip-fourth').setMinValue(fieldItems.get('ip-fourth').getValue() + 1);
						destFieldItems.get('ip-fourth').validate();

						if (destFieldItems.get('ip-fourth').isValid()) {
							me.fireEvent('rangechange', me, me.getSubnetRange());
						}
					}
				}
			},
			{
				xtype: 'label',
				text: '-',
				margin: '2 6 2 6',
			},
			{
				xtype: 'asVpnIPField',
				itemId: 'destination',
				width: 180,
				minMaxConf: me.destMinMaxConf,
				listeners: {
					change: function (field) {
						var fieldItems = field.items,
							destFieldItems = field.up().items.get('destination').items;

						if (destFieldItems.get('ip-fourth').isValid()) {
							me.fireEvent('rangechange', me, me.getSubnetRange());
						}
					}
				}
			}
		];

		me.callParent(arguments);

		me.on('afterrender', me.handleafterrender);
	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);

		me.un('afterrender', me.handleafterrender);
	},

	handleafterrender: function () {
		var me = this,
			origItems = me.items.get('original').items,
			destItems = me.items.get('destination').items;

		me.items.get('original').setAddress(me.defaultAddress[0]);
		me.items.get('destination').setAddress(me.defaultAddress[1]);

		var dValue = me.fieldDisabled;

		if (me.fieldDisabled.length !== 0) {
			origItems.get('ip-first').setDisabled(dValue[0][0]);
			origItems.get('ip-second').setDisabled(dValue[0][1]);
			origItems.get('ip-third').setDisabled(dValue[0][2]);
			origItems.get('ip-fourth').setDisabled(dValue[0][3]);

			destItems.get('ip-first').setDisabled(dValue[1][0]);
			destItems.get('ip-second').setDisabled(dValue[1][1]);
			destItems.get('ip-third').setDisabled(dValue[1][2]);
			destItems.get('ip-fourth').setDisabled(dValue[1][3]);
		} else {			
			destItems.get('ip-first').disable();
			destItems.get('ip-second').disable();
			destItems.get('ip-third').disable();			
		}
	},

	getValue: function () {
		var me = this,
			original = me.items.get('original'),
			dest = me.items.get('destination');

		return [original.getValue(), dest.getValue()];
	},

	setValue: function (start, end) {
		var me = this,
			original = me.items.get('original'),
			dest = me.items.get('destination');

		original.setAddress(start);
		dest.setAddress(end);
	}
});

Ext.define('AS.ARC.apps.vpnServer.core.IPField', {
	extend: 'Ext.container.Container',

	alias: ['widget.asVpnIPField'],

	layout: {type: 'hbox', align: 'middle'},

	cls: 'asVpnIPField',

	defaultAddress: '',

	ipFields: [],

	minMaxConf: [
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 0, max: 255 },
		{ min: 0, max: 255 }
	],

	initComponent: function () {
		var me = this;

		me.items = [
			{
				xtype: 'numberfield',
				itemId: 'ip-first',
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011'),
				allowDecimals: false,
				hideTrigger: true,
				maxValue: me.minMaxConf[0].max,
				minValue: me.minMaxConf[0].min,
				width: 36,
				listeners: {
					change: function (me) {
						me.up().fireEvent('change', me.up());
					}
				}
			},
			{
				xtype: 'displayfield',
				margin: '0 2 0 2',
				value: '.',
				cls: 'vpn-ip-dot'
			},
			{
				xtype: 'numberfield',
				itemId: 'ip-second',
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011'),
				allowDecimals: false,
				hideTrigger: true,
				maxValue: me.minMaxConf[1].max,
				minValue: me.minMaxConf[1].min,
				width: 36,
				listeners: {
					change: function (me) {
						me.up().fireEvent('change', me.up());
					}
				}
			},
			{
				xtype: 'displayfield',
				margin: '0 2 0 2',
				value: '.',
				cls: 'vpn-ip-dot'
			},
			{
				xtype: 'numberfield',
				itemId: 'ip-third',
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011'),
				allowDecimals: false,
				hideTrigger: true,
				maxValue: me.minMaxConf[2].max,
				minValue: me.minMaxConf[2].min,
				width: 36,
				listeners: {
					change: function (me) {
						me.up().fireEvent('change', me.up());
					}
				}
			},
			{
				xtype: 'displayfield',
				margin: '0 2 0 2',
				value: '.',
				cls: 'vpn-ip-dot'
			},
			{
				xtype: 'numberfield',
				itemId: 'ip-fourth',
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011'),
				allowDecimals: false,
				hideTrigger: true,
				maxValue: me.minMaxConf[3].max,
				minValue: me.minMaxConf[3].min,
				width: 36,
				listeners: {
					change: function (me) {
						me.up().fireEvent('change', me.up());
					}
				}
			}
		];

		me.callParent(arguments);

		me.on('afterrender', me.handleafterrender);

		me.addEvents('change');
	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);

		me.un('afterrender', me.handleafterrender);
	},

	setAddress: function (ip) {
		var ip = ip.split('.'),
			me = this,
			items = me.items;

		items.get('ip-first').clearInvalid();
		items.get('ip-second').clearInvalid();
		items.get('ip-third').clearInvalid();
		items.get('ip-fourth').clearInvalid();

		items.get('ip-first').setValue(ip[0]);
		items.get('ip-second').setValue(ip[1]);
		items.get('ip-third').setValue(ip[2]);
		items.get('ip-fourth').setValue(ip[3]);
	},

	getValue: function () {
		var me = this,
			ip = [],
			items = me.items;
		ip.push(items.get('ip-first').getValue());
		ip.push(items.get('ip-second').getValue());
		ip.push(items.get('ip-third').getValue());
		ip.push(items.get('ip-fourth').getValue());
		return ip.join('.');
	},

	handleafterrender: function () {
		var me = this;
		me.setAddress(me.defaultAddress);
	}
});

Ext.define('AS.ARC.apps.vpnServer.core.dnsIPField', {
	extend: 'Ext.container.Container',

	alias: ['widget.asVpnDnsIPField'],

	layout: 'hbox',

	// style: 'margin: 10 0 0 0',

	defaultAddress: '',

	initComponent: function () {
		var me = this;

		me.items = [
			{
				xtype: 'checkbox',
				itemId: 'checked',
				boxLabel: _S('NETWORK', 'DNS_SERVER'),
				width: 245,
				padding: '5 0 0 0',
				listeners: {
					change: function (me, newValue) {
						//if (newValue)
						//	me.up().items.get('dns-ct').enable();
						//else
						//	me.up().items.get('dns-ct').disable();
					}
				}
			},
			{
				xtype: 'container',
				itemId: 'dns-ct',
				width: 250,
				items: [
					{
						xtype: 'asVpnIPField',
						itemId: 'ip',
						defaultAddress: '0.0.0.0'
					}
				]
			}
		];

		me.callParent(arguments);

		me.on('afterrender', me.handleafterrender);
	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);

		me.un('afterrender', me.handleafterrender);
	},

	setAddress: function (ip) {
		var me = this,
			dns= me.items.get('dns-ct').items.get('ip');
		dns.setAddress(ip);
	},

	setChecked: function (value) {
		var me = this;
		me.items.get('checked').setValue(value);
	},

	getChecked: function () {
		var me = this;
		return me.items.get('checked').getValue();
	},

	getAddress: function () {
		var me = this,
			dns = me.items.get('dns-ct').items.get('ip');
		return dns.getValue();
	},

	handleafterrender: function () {
		var me = this;
		me.setAddress(me.defaultAddress);
	}
});

Ext.define('AS.ARC.apps.vpnServer.core.portField', {
	extend: 'Ext.container.Container',

	alias: ['widget.asVpnPortField'],

	layout: 'hbox',

	cls: 'vpn-btn',

	style: 'margin: 10px 0px;',

	defaultPort: '',

	initComponent: function () {
		var me = this;

		me.items = [
			{
				xtype: 'numberfield',
				itemId: 'port',
				allowDecimals: false,
				minValue: 1,
				maxValue: 65535,
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011'),
				//width: 250,
				fieldLabel: _S('COMMON', 'PORT'),
				labelWidth: AS.ARC.config.layout.labelWidthMedium,
				labelSeparator: ''
			},
			{
				xtype: 'button',
				text: _S('VPN_SERVER', 'USE_DEFAULT_PORT'),
				margin: '0 0 0 5',
				handler: function (me) {
					var portCt = me.up();
					portCt.items.get('port').setValue(portCt.defaultPort);
				}
			}
		];

		me.callParent(arguments);

		me.on('afterrender', me.handleafterrender);
	},

	setValue: function (value) {
		var me = this;
		me.items.get('port').setValue(value);
	},

	getValue: function () {
		var me = this;
		return me.items.get('port').getValue();
	},

	beforeDestroy: function () {
		var me = this;

		me.callParent(arguments);
		me.un('afterrender', me.handleafterrender);
	},

	handleafterrender: function () {
		var me = this;
		me.items.get('port').setValue(me.defaultPort);
	}

});

Ext.define('AS.ARC.apps.vpnServer.settings.general', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'general',

    resetForm: function (form) {
		var me = this;

		form.getFields().each(function (f) {
			if (f.resetOriginalValue) {
				f.resetOriginalValue();
			}
		});
		form.checkDirty();
	},
	
	handleApply: function () {
		var page = this;

		var interfaceData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(interfaceData)) {
			page.fp.showSetNotChange();
			return false;
		}

		page.fn.app.maskWin(_S('COMMON', 'LOADING'));
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
				act: 'set',
				section: 'settings',
				tab: 'general'
			}),
			params: interfaceData,
			method: 'post',
			success: function (json, opts) {
				page.fn.app.unmaskWin();
				page.general.loadData();
			},
			failure: function (json, opts) {
				page.fn.app.unmaskWin();
				AS.ARC.util.showMsgWindow({

				}, json, page.fn.app.win);
			}
		});
	},
    
	getPanel: function () {
		var page = this;

        page.netInt = Ext.create(Ext.form.ComboBox, {
			itemId: 'net-int',
            msgTarget: AS.ARC.config.msgTarget,
			editable: false,
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			fieldLabel:  _S('VPN_SERVER', 'INTERFACE'),
			store: Ext.create(Ext.data.ArrayStore, {
				fields: ['name', 'value']
			}),
			displayField: 'name',
			valueField: 'value',
			queryMode: 'local',
			lostInt: '',
			toDisplay: function (net) {
				var result = net.name;

				if (net.name.toLowerCase() === 'wifi1') {
					result = _S('COMMON', 'WIFI');
				} else if (net.name.toLowerCase() === 'pppoe') {
					result = _UN_TS('PPPoE');
				} else if (net.display.split(',').length === 1) {
					result = 'LAN' + ' ' + net.display[3]; 
				} else {
					result = 'LAN ' + net.display.split(',').join('+'); // for link aggregation
				}

				return result;
			},
            validator: function (value) {
                // value: getRawValue(), displayField value
                var me = this;
                // me.value: combo value
                if (me.value === me.lostInt) {
					if (me.getDisplayValue() === '\u00a0') {
						return _S('VPN_SERVER', 'SELECT_INTERFACE');
					} else {
						return Ext.String.format(_S('VPN_SERVER', 'LAN_LOST'), me.getDisplayValue());
					}
                } else {
                    return true;
                } 
            }
		});

		page.autoBlock = Ext.create(Ext.form.DisplayField, {
			itemId: 'dp-autoBlock',
			fieldCls: 'green-string',
			width: 370,
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			fieldLabel: _S('DEFENDER', 'NETWORK_DEFENDER'),
			value: 'Enabled'
		});

		page.autoBlockLink = Ext.create(Ext.form.DisplayField, {
			itemId: 'dp-autoBlockLink',
			fieldCls: 'hyperlink-string',
			fieldStyle: 'cursor:pointer;text-decoration:underline;',
			width: 300,
			value: '(' + _S('VPN_SERVER', 'SETUP') + ')',
			margin: '0 0 0 7',
			listeners: {
				afterrender: function (me) {
					me.el.on('click', function () {
						AS.ARC.core.openApp('app-settings', 'defender', 0);
					});
				}
			}
		});

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		page.general = Ext.create('AS.ARC.panelTop10', {
            padding: 0,
			itemId: 'general-tab',
			page: page,
			cls: 'as-page-panel',
            border: 'none',
            items: [
				{
					xtype: 'fieldset',
					title: _S('COMMON', 'GENERAL'),
					items: [
						page.netInt,
						{
							xtype: 'container',
							layout: 'hbox',
							items: [
								page.autoBlock,
								page.autoBlockLink
							]
						}
					]
				}
			],
			loadData: function () {
				var me = this;
				page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'get',
						section: 'settings',
						tab: 'general'
					}),
					method: 'post',
					success: function (json, opts) {
						var list = [],
							existInterface = false;
							
						Ext.each(json.net, function (data) {
							list.push({
								name: page.netInt.toDisplay(data),
								value: data.name
							});
                            if (json['interface'] === data.name) {
                                existInterface = true;
                            }
						});
                        
						//got lost interface
                        if (!existInterface) {
							page.netInt.lostInt = json['interface'];
                            list.push({
								// 如果後端回傳的 display 欄位是空值，就顯示 nonbreaking space 字元，避免該選項高度錯誤
                            	name: json['display'] === '' ? '\u00a0' : page.netInt.toDisplay({ name: json['interface'], display: json['display'] }),
								value: json['interface']
							});
                        }
                        
                        page.netInt.store.removeAll();
						page.netInt.store.add(list);
						page.netInt.setValue(json['interface']);
						
						if (json.ipblock) {
							page.autoBlock.setValue(_S('COMMON', 'ACTIVE'));
							page.autoBlock.inputEl.removeCls('red-string').addCls('green-string');
						} else {
							page.autoBlock.setValue(_S('COMMON', 'DISABLED'));
							page.autoBlock.inputEl.removeCls('green-string').addCls('red-string');
						}

						page.resetForm(page.general.getForm());		

						page.fp.uiChecker.srcData = page.doDataExchange();

						page.fn.app.unmaskWin();
					},
					failure: function (json, opts) {
						page.fn.app.unmaskWin();
						AS.ARC.util.showMsgWindow({

						}, json, page.fn.app.win);
					}
				});
			}
		});

		page.doDataExchange = function () {
			return {
				'interface': page.netInt.getValue()
			};
		};
        
		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title: _S('COMMON', 'GENERAL'),
			border: false,
			cls: 'as-page-panel',
			items: [
				page.general
            ],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange
		};
	},

	loadData: function () {
		var page = this;
		page.general.loadData();
	}
});

Ext.define('AS.ARC.apps.vpnServer.settings.pptp', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'pptp',

    resetForm: function (form) {
		var me = this;

		form.getFields().each(function (f) {
			if (f.resetOriginalValue) {
				f.resetOriginalValue();
			}
		});
		form.checkDirty();
	},

	handleApply: function () {
		var page = this;

		var pptpData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(pptpData)) {
			page.fp.showSetNotChange();
			return false;
		}

		var doApply = function () {
			page.fn.app.maskWin(_S('COMMON', 'LOADING'));
			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
					act: 'set',
					section: 'settings',
					tab: 'pptp'
				}),
				params: pptpData,
				method: 'post',
				success: function (json, opts) {
					page.fn.app.unmaskWin();
					page.pptp.loadData();
					page.fp.showApply();
				},
				failure: function (json, opts) {
					page.fn.app.unmaskWin();
					page.pptp.loadData();
					AS.ARC.util.showMsgWindow({
						5804: _S('VPN_SERVER', 'IP_USED')
					}, json, page.fn.app.win);
				}
			});
		};
		page.confirmEnable(page.fn.app.win, _S('VPN_SERVER', 'PPTP'), doApply);
	},

	getPanel: function () {
		var page = this;

		// 彈出確認視窗
		page.confirmEnable = function (parentWin, type, cb) {
			var win = Ext.create('AS.ARC.msgWindow', {
				title: _S('COMMON', 'CONFIRMATION'),
				parentWin: parentWin,
				height: 200,
				iconType: 'question', // info, question, warning
				asItems: [
					{
						xtype: 'displayfield',
						value: Ext.String.format(_S('VPN_SERVER', 'SERVER_SWITCH_CONFIRM'), type)
					}
				],
				scope: this,
				fbar: [{
					text: _S('COMMON', 'OK'),
					handler: function () {
						win.close();
						if (Ext.isFunction(cb)) cb();
					},
				}, {
					text: _S('COMMON', 'CANCEL'),
					handler: function () {
						win.close();						
					}
				}]
			}).show();
		};

		page.enablePPTP = Ext.create(Ext.form.Checkbox, {
            itemId: 'enablePPTP',
			boxLabel: _S('COMMON', 'ENABLE'),
			labelWidth: AS.ARC.config.layout.defaultLabelWidth,
			setMask: function (newValue) {
				if (newValue) {
					page.pptp.items.get('fs').items.get('ct').enable();
				} else {
					page.pptp.items.get('fs').items.get('ct').disable();
				}
			},
			listeners: {
				change: function (me, newValue) {
					var auth = me.up().down('[itemId=pptp-auth]').getValue(),
					    encryp = me.up().down('[itemId=pptp-encryp]');

					if (auth === 'PAP') {
						encryp.disable();
					} else {
						encryp.enable();
					}

					me.setMask(newValue);
				},
				dirtychange: function(me, isDirty) {
					var auth = me.up().down('[itemId=pptp-auth]').getValue(),
					    encryp = me.up().down('[itemId=pptp-encryp]');

					if (auth === 'PAP') {
						encryp.disable();
					} else {
						encryp.enable();
					}
				}
			}
		});

		page.pptpDynIp = Ext.create('AS.ARC.apps.vpnServer.core.dynIPField', {
            itemId: 'pptpDynIp',
			fieldDisabled: [[false, false, false, false], [true, true, true, false]],
			defaultAddress: ['10.0.0.2', '10.0.0.254'],
			//margin: '10 0 0 0',
			//height: 25,
			origMinMaxConf: [
				{ min: 0, max: 255 },
				{ min: 0, max: 255 },
				{ min: 0, max: 255 },
				{ min: 2, max: 255 }
			],
			//destMinMaxConf: [
			//	{ min: 0, max: 255 },
			//	{ min: 0, max: 255 },
			//	{ min: 0, max: 255 },
			//	{ min: 2, max: 254 }
			//],
			listeners: {
				rangechange: function (me, value) {
					page.pptpMaxClient.validate();
				}
			}
		});

		page.pptpMaxClient = Ext.create('AS.ARC.apps.vpnServer.core.maxClientCombo', {
            itemId: 'pptpMaxClient',
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'MAX_CLIENT_NUMBER'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			//margin: '10 0 0 0',
			msgTarget: AS.ARC.config.msgTarget,
			validator: function (value) {
				page.pptpDynIp.getValue();

				if (page.pptpDynIp.getSubnetRange() < value) {
					return _S('VPN_SERVER', 'INVALID_MAX_CLIENT');
				} else {
					return true;
				}
			}
		});

		page.pptpAuth = Ext.create(Ext.form.ComboBox, {
			labelSeparator: '',
			editable: false,
			fieldLabel: _S('COMMON', 'AUTHENTICATION'),
			itemId: 'pptp-auth',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			queryMode: 'local',
			displayField: 'name',
			valueField: 'value',
			value: 5,
			store: Ext.create(Ext.data.Store, {
				fields: ['name', 'value'],
				data: [
					{ name: _S('VPN_SERVER', 'PAP'), value: 'PAP' },
					{ name: _S('VPN_SERVER', 'MSCHAPV2'), value: 'MSCHAPv2' }
				]
			}),
			listeners: {
				change: function (me, newValue) {
					if (newValue === 'PAP') {
						page.pptpEncryp.disable();
					} else {
						page.pptpEncryp.enable();
					}
				}
			}
		});

		page.pptpEncryp = Ext.create(Ext.form.ComboBox, {
			labelSeparator: '',
			editable: false,
			fieldLabel: _S('COMMON', 'ENCRYPTION'),
			itemId: 'pptp-encryp',
			//margin: '10 0 0 0',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			queryMode: 'local',
			displayField: 'name',
			valueField: 'value',
			value: 5,
			store: Ext.create(Ext.data.Store, {
				fields: ['name', 'value'],
				data: [
					{ name: _S('COMMON', 'NONE'), value: 'None' },
					{ name: _S('VPN_SERVER', 'MPPE'), value: 'MPPE' },
					{ name: _S('VPN_SERVER', 'MAX_MPPE'), value: 'MaxMPPE' }
				]
			})
		});

		page.pptpMTU = Ext.create(Ext.form.NumberField, {
            itemId: 'pptpMTU',
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'MAX_MTU'),
			//margin: '10 0 0 0',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			minValue: 68,
			maxValue: 1500,
			value: 1400,
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			msgTarget: AS.ARC.config.msgTarget
		});
		
		page.pptpMRU = Ext.create(Ext.form.NumberField, {
            itemId: 'pptpMRU',
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'MAX_MRU'),
			//margin: '10 0 0 0',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			minValue: 68,
			maxValue: 1500,
			value: 1400,
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			msgTarget: AS.ARC.config.msgTarget
		});
	
		page.pptpDNS = Ext.create('AS.ARC.apps.vpnServer.core.dnsIPField', {
            itemId: 'pptpDNS',
			defaultAddress: '0.0.0.0',
		});

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		page.pptp = Ext.create('AS.ARC.panelTop10', {
            padding: 0,
			itemId: 'pptp-tab',
			page: page,
			cls: 'as-page-panel',
            border: 'none',
			items: [
				{
					xtype: 'fieldset',
					itemId: 'fs',
					padding: '10 10 10 10',
					title: _S('COMMON', 'GENERAL'),
					items: [
						page.enablePPTP,
						{
							xtype: 'container',
							itemId: 'ct',
							disabled: true,
							style: 'padding-left: 10px;',
							cls: 'vpnserver-mask',
							items: [
								page.pptpDynIp,
								page.pptpMaxClient,
								page.pptpAuth,
								page.pptpEncryp,
								page.pptpMTU,
								page.pptpMRU,
								page.pptpDNS
							]
						}
					]
				}
			],
			loadData: function () {
				var me = this;
				page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'get',
						section: 'settings',
						tab: 'pptp'
					}),
					method: 'post',
					success: function (json, opts) {
						page.fn.app.unmaskWin();
						page.enablePPTP.setValue(json.enableserver);
						page.pptpDynIp.setValue(json.ipstart, json.ipend);
						page.pptpMaxClient.setValue(json.maxclient);
						page.pptpAuth.setValue(json.auth);
						page.pptpEncryp.setValue(json.enc);
						page.pptpMTU.setValue(json.mtu);
						page.pptpMRU.setValue(json.mru);
						page.pptpDNS.setChecked(json.enabledns);
						page.pptpDNS.setAddress(json.ipdns);
						page.enablePPTP.setMask(json.enableserver);

						page.resetForm(page.pptp.getForm());
						page.fp.uiChecker.srcData = page.doDataExchange();
					},
					failure: function (json, opts) {
						page.fn.app.unmaskWin();
						AS.ARC.util.showMsgWindow({
						}, json, page.fn.app.win);
					}
				});
			},
			listeners: {
				show: function () {
					pptpChangedLabel.resetDisplay();
				}
			}
		});

		page.doDataExchange = function () {
			return {
				enableserver: page.enablePPTP.getValue(),
				ipstart: page.pptpDynIp.getValue()[0],
				ipend: page.pptpDynIp.getValue()[1],
				maxclient: page.pptpMaxClient.getValue(),
				auth: page.pptpAuth.getValue(),
				enc: page.pptpEncryp.getValue(),
				mtu: page.pptpMTU.getValue(),
				mru: page.pptpMRU.getValue(),
				enabledns: page.pptpDNS.getChecked(),
				ipdns: page.pptpDNS.getAddress()
			};
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title:  _S('VPN_SERVER', 'PPTP'),
			border: false,
			cls: 'as-page-panel',
			items: [
				page.pptp
            ],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange
		};
	},

	loadData: function () {
		var page = this;
		page.pptp.loadData();
	}
});

Ext.define('AS.ARC.apps.vpnServer.settings.openvpn', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'openvpn',

	resetForm: function (form) {
		var me = this;

		form.getFields().each(function (f) {
			if (f.resetOriginalValue) {
				f.resetOriginalValue();
			}
		});
		form.checkDirty();
	},

	handleApply: function () {
		var page = this;

		var openVPNData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(openVPNData)) {
			page.fp.showSetNotChange();
			return false;
		}

		var doApply = function () {
			page.fn.app.maskWin(_S('COMMON', 'LOADING'));
			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
					act: 'set',
					section: 'settings',
					tab: 'openvpn'
				}),
				params: openVPNData,
				method: 'post',
				success: function (json, opts) {
					page.fn.app.unmaskWin();
					page.openVpn.loadData();
					page.fp.showApply();
				},
				failure: function (json, opts) {
					page.fn.app.unmaskWin();
					page.openVpn.loadData();
					AS.ARC.util.showMsgWindow({
						5032: _S('ERROR', 'PORT_USED'),
						5804: _S('VPN_SERVER', 'IP_USED')
					}, json, page.fn.app.win);
				}
			});
		};
		
		page.confirmEnable(page.fn.app.win, _S('VPN_SERVER', 'OPENVPN'), doApply);
	},

	getPanel: function () {
		var page = this;

        page.confirmEnable = function (parentWin, type, cb) {
			var win = Ext.create('AS.ARC.msgWindow', {
				title: _S('COMMON', 'CONFIRMATION'),
				parentWin: parentWin,
				height: 200,
				iconType: 'question', // info, question, warning
				asItems: [
					{
						xtype: 'displayfield',
						value: Ext.String.format(_S('VPN_SERVER', 'SERVER_SWITCH_CONFIRM'), type)
					}
				],
				scope: this,
				fbar: [{
					text: _S('COMMON', 'OK'),
					handler: function () {
						win.close();
						if (Ext.isFunction(cb)) cb();
					},
				}, {
					text: _S('COMMON', 'CANCEL'),
					handler: function () {
						win.close();						
					}
				}]
			}).show();
		};

		page.enableVPN = Ext.create(Ext.form.Checkbox, {
			boxLabel: _S('COMMON', 'ENABLE'),
			labelWidth: AS.ARC.config.layout.defaultLabelWidth,
			setMask: function (newValue) {
				if (newValue) {
					page.openVpn.items.get('fs').items.get('ct').enable();
				} else {
					page.openVpn.items.get('fs').items.get('ct').disable();
				}
			},
			listeners: {
				change: function (me, newValue) {
					me.setMask(newValue);
				}
			}

		});

		page.vpnDynIp = Ext.create('AS.ARC.apps.vpnServer.core.dynIPField', {
			fieldDisabled: [[false, false, false, true], [true, true, true, true]],
			defaultAddress: ['10.8.0.1', '10.8.0.254'],
		});

		page.vpnTransProtocol = Ext.create(Ext.form.ComboBox, {
			labelSeparator: '',
			editable: false,
			fieldLabel: _S('VPN_SERVER', 'TRANS_PROTOCOL'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			queryMode: 'local',
			displayField: 'name',
			valueField: 'value',
			value: 0,
			store: Ext.create(Ext.data.Store, {
				fields: ['name', 'value'],
				data: [
					{ name: _S('DEFENDER', 'TCP'), value: 'TCP' },
					{ name: _S('DEFENDER', 'UDP'), value: 'UDP' }
				]
			})
		});

		page.vpnPort = Ext.create('AS.ARC.apps.vpnServer.core.portField', {
			defaultPort: 1194,
			//margin: '8 0 0 0',
			//height: 25
		});

		page.vpnMaxClient = Ext.create('AS.ARC.apps.vpnServer.core.maxClientCombo', {
			labelSeparator: '',
			margin: '10 0 0 0',
			fieldLabel: _S('VPN_SERVER', 'MAX_CLIENT_NUMBER'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium
		});

		page.vpnAuth = Ext.create(Ext.form.ComboBox, {
			labelSeparator: '',
			editable: false,
			fieldLabel: _S('COMMON', 'CHECKSUM') + ' (' + _S('VPN_SERVER', 'DIGEST') + ')',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			//margin: '8 0 0 0',
			queryMode: 'local',
			displayField: 'name',
			valueField: 'value',
			value: 5,
			store: Ext.create(Ext.data.Store, {
				fields: ['name', 'value'],
				data: [
					{ name: _S('COMMON', 'NONE'), value: 'None' },
					{ name: _UN_TS('SHA1'), value: 'SHA1' },
					{ name: _UN_TS('SHA256'), value: 'SHA256' },
					{ name: _UN_TS('SHA512'), value: 'SHA512' },
					{ name: _UN_TS('MD4'), value: 'MD4' },
					{ name: _UN_TS('MD5'), value: 'MD5' }
				]
			})
		});

		page.vpnEncryp = Ext.create(Ext.form.ComboBox, {
			labelSeparator: '',
			editable: false,
			fieldLabel: _S('COMMON', 'ENCRYPTION') + ' (' + _S('VPN_SERVER', 'CIPHER') + ')',
			//margin: '8 0 0 0',
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			queryMode: 'local',
			displayField: 'name',
			valueField: 'value',
			value: 0,
			store: Ext.create(Ext.data.Store, {
				fields: ['name', 'value'],
				data: [
					{ name: _S('COMMON', 'NONE'), value: 'None' },
					{ name: _UN_TS('BF-CBC'), value: 'BF-CBC' },
					{ name: _UN_TS('AES-128-CBC'), value: 'AES-128-CBC' },
					{ name: _UN_TS('AES-192-CBC'), value: 'AES-192-CBC' },
					{ name: _UN_TS('AES-256-CBC'), value: 'AES-256-CBC' }
				]
			})
		});
		
		page.vpnDNS = Ext.create('AS.ARC.apps.vpnServer.core.dnsIPField', {
			defaultAddress: '0.0.0.0',
			//margin: '10 0 0 0',
			//height: 25
		});

		page.vpnRedGate = Ext.create(Ext.form.Checkbox, {
			boxLabel: _S('VPN_SERVER', 'ENABLE_REDIRECT'),
			//margin: '8 0 0 0'
		});

		page.vpnLink = Ext.create(Ext.form.Checkbox, {
			boxLabel: _S('VPN_SERVER', 'VPN_LINK'),
			//margin: '8 0 0 0'
		});

		page.vpnNewCertificate = Ext.create(Ext.button.Button, {
			text: _S('CERTIFICATE', 'CREATE_A_NEW_CERTIFICATE'),
			handler: function () {
				var win = Ext.create('AS.ARC.msgWindow', {
					title: _S('COMMON', 'CONFIRMATION'),
					parentWin: page.fn.app.win,
					height: 200,
					iconType: 'question',
					asItems: [
						{
							xtype: 'displayfield',
							value: Ext.String.format(_S('VPN_SERVER', 'SERVER_CERTIFICATE_CONFIRM'), _S('VPN_SERVER', 'OPENVPN'))
						}
					],
					scope: this,
					fbar: [{
						text: _S('COMMON', 'OK'),
						handler: function () {
							page.fn.app.maskWin(_S('COMMON', 'APPLYING'));
							AS.ARC.ajax({
								url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
									act: 'new_cert',
									section: 'settings',
									tab: 'openvpn'
								}),
								success: function() {
									page.vpnDownload.handler();
									page.fn.app.unmaskWin();
								},
								failure: function(json) {
									page.fn.app.unmaskWin();
									AS.ARC.util.showMsgWindow({
									}, json, page.fn.app.win)
								}
							});

							win.close();
						},
					}, {
						text: _S('COMMON', 'CANCEL'),
						handler: function () {
							win.close();						
						}
					}]
				}).show();
			}
		});

		page.vpnDownload = Ext.create(Ext.button.Button, {
			text: _S('VPN_SERVER', 'DOWNLOAD_CONFG_FILE'),
			handler: function (me) {
				var doDownload = function (filename) {
					var formPanel = Ext.create('Ext.form.FormPanel', {
						title: '',
						hidden: true,
						items: [{
							xtype: 'filefield'
						}]
					});

					formPanel.form.submit({
						url: AS.ARC.util.getApiUrlWithSid(AS.ARC.util.getApiUrl('download.cgi', 'fileExplorer'), { act: 'download_sys_setting' }) + '&file1=' + filename,
						success: function (f, e) {

						},
						failure: function (form, action) {
							var json = Ext.decode(action.response.responseText);

							AS.ARC.util.showMsgWindow({
								5301: _S('ERROR', 'UNEXPECTED_ACTION'),
								5302: _S('ERROR', 'INCORRECT_PARAMETER'),
								5011: _S('ERROR', 'CANNOT_CREATE_TMP_DIRECTORY'),
								5005: _S('ERROR', 'UNABLE_EXPORT_SETTINGS'),
								5025: _S('ERROR', 'UNABLE_IMPORT_SETTINGS_ACCESS_RIGHT')
							}, json, page.fn.app.win);
						}
					});
				}
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'download',
						section: 'settings',
						tab: 'openvpn'
					}),
					method: 'post',
					success: function (json, opts) {
						doDownload(json.filename);
					},
					failure: function (json, opts) {
						AS.ARC.util.showMsgWindow({

						}, json, page.fn.app.win);
					}
				});								
			}
		});

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		page.openVpn = Ext.create('AS.ARC.panelTop10', {
			itemId: 'openvpn-tab',
            padding: 0,
			page: page,
			cls: 'as-page-panel',
            border: 'none',
			items: [
				{
					xtype: 'fieldset',
					itemId: 'fs',
					title: _S('COMMON', 'GENERAL'),
					items: [
						page.enableVPN,
						{
							xtype: 'container',
							itemId: 'ct',
							disabled: true,
							cls: 'vpnserver-mask',
							style: 'padding-left: 10px;',
							items: [
								page.vpnDynIp,
								page.vpnTransProtocol,
								page.vpnPort,
								page.vpnMaxClient,
								page.vpnAuth,
								page.vpnEncryp,
								page.vpnDNS,
								page.vpnRedGate,
								page.vpnLink
							]
						}
					]
				},
				{
					xtype: 'fieldset',
					title: _S('VPN_SERVER', 'CONFG_FILE_DOWNLOAD'),
					items: [
						{
							xtype: 'container',
							layout: 'hbox',
							cls: 'vpn-btn',
							items: [
								page.vpnNewCertificate,
								page.vpnDownload,
								{
									xtype: 'component',
									style: 'width: 28px; height: 28px;',
									html: '<img src="' + page.fn.imageUrl + 'icon-information.png' + '" data-qtip="' + _S('VPN_SERVER', 'DOWNLOAD_INFO') + '" style="width: 16px; position: absolute; top: 0; left: 0; right: 0; bottom: 0; margin: auto;">'
								}
							]
						}						
					]
				},
			],
			loadData: function () {
				var me = this;
				page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'get',
						section: 'settings',
						tab: 'openvpn'
					}),
					method: 'post',
					success: function (json, opts) {
						page.enableVPN.setValue(json.enableserver);
						page.vpnDynIp.setValue(json.ipstart, json.ipend);
						page.vpnTransProtocol.setValue(json.protocol);
						page.vpnPort.setValue(json.port);
						page.vpnMaxClient.setValue(json.maxclient);
						page.vpnAuth.setValue(json.auth);
						page.vpnEncryp.setValue(json.cipher);
						page.vpnDNS.setChecked(json.enabledns);
						page.vpnDNS.setAddress(json.ipdns);
						page.vpnRedGate.setValue(json.gateway);
						page.vpnLink.setValue(json.complzo);
						page.enableVPN.setMask(json.enableserver);

						page.resetForm(page.openVpn.getForm());

						page.fp.uiChecker.srcData = page.doDataExchange();

						page.fn.app.unmaskWin();
					},
					failure: function (json, opts) {
						page.fn.app.unmaskWin();
						AS.ARC.util.showMsgWindow({
                            
						}, json, page.fn.app.win);
					}
				});
			}
		});
        
		page.doDataExchange = function () {
			return {
				enableserver: page.enableVPN.getValue(),
				ipstart: page.vpnDynIp.getValue()[0],
				ipend: page.vpnDynIp.getValue()[1],
				protocol: page.vpnTransProtocol.getValue(),
				port: page.vpnPort.getValue(),
				maxclient: page.vpnMaxClient.getValue(),
				auth: page.vpnAuth.getValue(),
				cipher: page.vpnEncryp.getValue(),
				enabledns: page.vpnDNS.getChecked(),
				ipdns: page.vpnDNS.getAddress(),
				gateway: page.vpnRedGate.getValue(),
				complzo: page.vpnLink.getValue()
			};
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title:  _S('VPN_SERVER', 'OPENVPN'),
			border: false,
			cls: 'as-page-panel',
			items: [
				page.openVpn
            ],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange
		};
	},

	loadData: function () {
		var page = this;
		page.openVpn.loadData();
	}
});

Ext.define('AS.ARC.apps.vpnServer.settings.xl2tp', {
	extend: 'AS.ARC.apps.vpnServer.page',
	pageTag: 'xl2tp',

    resetForm: function (form) {
		var me = this;

		form.getFields().each(function (f) {
			if (f.resetOriginalValue) {
				f.resetOriginalValue();
			}
		});
		form.checkDirty();
	},

	handleApply: function () {
		var page = this;

		var xl2tpData = page.fp.doDataExchange();
		if (!page.fp.isSettingChange(xl2tpData)) {
			page.fp.showSetNotChange();
			return false;
		}

		var doApply = function () {
			page.fn.app.maskWin(_S('COMMON', 'LOADING'));
			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
					act: 'set',
					section: 'settings',
					tab: 'xl2tp'
				}),
				params: xl2tpData,
				method: 'post',
				success: function (json, opts) {
					page.fn.app.unmaskWin();
					page.xl2tp.loadData();
					page.fp.showApply();
				},
				failure: function (json, opts) {
					page.fn.app.unmaskWin();
					page.xl2tp.loadData();
					AS.ARC.util.showMsgWindow({
						5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
						5302: _S('ERROR', 'INCORRECT_PARAMETER'),
						5006: _S('ERROR', 'CANNOT_READ_CONFIG')
					}, json, page.fn.app.win);
				}
			});
		};
		
		page.confirmEnable(page.fn.app.win, _S('VPN_SERVER', 'XL2TP'), doApply);
	},

	getPanel: function () {
		var page = this;

		page.confirmEnable = function (parentWin, type, cb) {
			var win = Ext.create('AS.ARC.msgWindow', {
				title: _S('COMMON', 'CONFIRMATION'),
				parentWin: parentWin,
				height: 200,
				iconType: 'question', // info, question, warning
				asItems: [
					{
						xtype: 'displayfield',
						value: Ext.String.format(_S('VPN_SERVER', 'SERVER_SWITCH_CONFIRM'), type)
					}
				],
				scope: this,
				fbar: [{
					text: _S('COMMON', 'OK'),
					handler: function () {
						win.close();
						if (Ext.isFunction(cb)) cb();
					},
				}, {
					text: _S('COMMON', 'CANCEL'),
					handler: function () {
						win.close();						
					}
				}]
			}).show();
		};

		page.enableXl2tp = Ext.create(Ext.form.Checkbox, {
			boxLabel: _S('COMMON', 'ENABLE'),
			labelWidth: AS.ARC.config.layout.defaultLabelWidth,
			setMask: function (newValue) {
				if (newValue) {
					page.xl2tp.items.get('fs').items.get('ct').enable();
				} else {
					page.xl2tp.items.get('fs').items.get('ct').disable();
				}
			},
			listeners: {
				change: function (me, newValue) {
					me.setMask(newValue);
				}
			}
		});

		page.xl2tpDynIp = Ext.create('AS.ARC.apps.vpnServer.core.dynIPField', {
			fieldDisabled: [[false, false, false, false], [true, true, true, false]],
			defaultAddress: ['10.0.0.2', '10.0.0.254'],
			origMinMaxConf: [
				{ min: 0, max: 255 },
				{ min: 0, max: 255 },
				{ min: 0, max: 255 },
				{ min: 2, max: 255 }
			],
			listeners: {
				rangechange: function (me, value) {
					page.xl2tpMaxClient.validate();
				}
			}
		});

		page.xl2tpMaxClient = Ext.create('AS.ARC.apps.vpnServer.core.maxClientCombo', {
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'MAX_CLIENT_NUMBER'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			msgTarget: AS.ARC.config.msgTarget,
			validator: function (value) {
				page.xl2tpDynIp.getValue();

				if (page.xl2tpDynIp.getSubnetRange() < value) {
					return _S('VPN_SERVER', 'INVALID_MAX_CLIENT');
				} else {
					return true;
				}
			}
		});

		page.xl2tpMTU = Ext.create(Ext.form.NumberField, {
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'MAX_MTU'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			minValue: 68,
			maxValue: 1500,
			value: 1400,
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			msgTarget: AS.ARC.config.msgTarget
		});
		
		page.xl2tpDNS = Ext.create('AS.ARC.apps.vpnServer.core.dnsIPField', {
			defaultAddress: '0.0.0.0',
		});
		
		page.xl2tpKey = Ext.create('Ext.form.field.Text', {
			labelSeparator: '',
			fieldLabel: _S('VPN_SERVER', 'KEY'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			msgTarget: AS.ARC.config.msgTarget,
			maxLength: 32,
			vtype: 'chapPassword',
		});

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		// xl2tp
		page.xl2tp = Ext.create('AS.ARC.panelTop10', {
			itemId: 'xl2tp-tab',
			padding: 0,
			page: page,
			cls: 'as-page-panel',
			border: 'none',
			items: [
				{
					xtype: 'fieldset',
					itemId: 'fs',
					padding: '10 10 10 10',
					title: _S('COMMON', 'GENERAL'),
					items: [
						page.enableXl2tp,
						{
							xtype: 'container',
							itemId: 'ct',
							disabled: true,
							style: 'padding-left: 10px;',
							cls: 'vpnserver-mask',
							items: [
								page.xl2tpDynIp,
								page.xl2tpMaxClient,
								page.xl2tpMTU,
								page.xl2tpDNS,
								page.xl2tpKey
							]
						}
					]
				}
			],
			loadData: function () {
				var me = this;
				page.fn.app.maskWin(_S('COMMON', 'LOADING'));
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.fn.apiUrl, {
						act: 'get',
						section: 'settings',
						tab: 'xl2tp'
					}),
					method: 'post',
					success: function (json, opts) {
						page.fn.app.unmaskWin();						
						page.enableXl2tp.setValue(json.enableserver);
						page.xl2tpDynIp.setValue(json.ipstart, json.ipend);
						page.xl2tpMaxClient.setValue(json.maxclient);
						page.xl2tpMTU.setValue(json.mtu);
						page.xl2tpDNS.setChecked(json.enabledns);
						page.xl2tpDNS.setAddress(json.ipdns);
						page.enableXl2tp.setMask(json.enableserver);
						page.xl2tpKey.setValue(json.key);

						page.resetForm(page.xl2tp.getForm());

						page.fp.uiChecker.srcData = page.doDataExchange();
					},
					failure: function (json, opts) {
						page.fn.app.unmaskWin();
						AS.ARC.util.showMsgWindow({
							5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
							5302: _S('ERROR', 'INCORRECT_PARAMETER'),
							5006: _S('ERROR', 'CANNOT_READ_CONFIG')
						}, json, page.fn.app.win);
					}
				});
			}
		});

		page.doDataExchange = function () {
			return {
				enableserver: page.enableXl2tp.getValue(),
				ipstart: page.xl2tpDynIp.getValue()[0],
				ipend: page.xl2tpDynIp.getValue()[1],
				maxclient: page.xl2tpMaxClient.getValue(),
				mtu: page.xl2tpMTU.getValue(),
				enabledns: page.xl2tpDNS.getChecked(),
				ipdns: page.xl2tpDNS.getAddress(),
				key: page.xl2tpKey.getValue()
			};
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId:  page.panelItemId,
			title:  _S('VPN_SERVER', 'XL2TP'),
			border: false,
			cls: 'as-page-panel',
			items: [
				page.xl2tp
            ],
			fbar: [
				{
					xtype:'tbfill'
				},
				page.buttonApply
			],
			doDataExchange: page.doDataExchange
		};
	},

	loadData: function () {
		var page = this;
		page.xl2tp.loadData();
	}
});

/**
 * @class AS.ARC.apps.vpnServer.settings.fn
 * @extends AS.ARC.apps.vpnServer.fn
 * <p></p>
 */
Ext.define('AS.ARC.apps.vpnServer.settings.fn', {
	extend: 'AS.ARC.apps.vpnServer.fn',

	fnTag: 'settings',
	
	init: function (config) {
		this.apiUrl = AS.ARC.util.getUserAppsPath() + 'vpn-server/vpnserver.cgi',
		this.imageUrl = AS.ARC.util.getUserAppsPath() + 'vpn-server/images/';
		
		this.createPanel();
		//this.loadData();
	},

	createPanel: function () {
		var fn = this;
		
		var pageGeneral = new AS.ARC.apps.vpnServer.settings.general({fn: this});
        var pagePptp = new AS.ARC.apps.vpnServer.settings.pptp({fn: this});
        var pageOpenvpn = new AS.ARC.apps.vpnServer.settings.openvpn({fn: this});
		var pageXl2tp = new AS.ARC.apps.vpnServer.settings.xl2tp({fn: this});
			
		this.tabPanel = Ext.create('AS.ARC.fnTabPanel', {
			items: [
				pageGeneral.getPanel(),
                pagePptp.getPanel(),
                pageOpenvpn.getPanel(),
				pageXl2tp.getPanel()
			],
			listeners: {
				tabchange: function(tabpanel) {
					var activeIndex = tabpanel.items.indexOf(tabpanel.getLayout().activeItem);
				}
			}
		});

		//fn.tabPanel.enableDataConfirm();
		fn.initMainPanel();
	},
	
	loadData: function () {
		this.tabPanel.activeTab.page.loadData(true);
	}
});